const shell = require('shelljs')
const path = require('path')
const _ = require('lodash')
const fse = require('fs-extra')

const version = '1.0.0'

class GitClone {
  constructor (options) {
    this.repositoryName = options.repositoryName
    this.remoteRepositoryPath = options.remoteRepositoryPath
    this.copyFile = options.copyFile
    this.projectPath = options.projectPath
    this.version = version
    this.initGit()
    // this.lastCommitHash = options.lastCommitHash
  }

  initGit () {
    if (!shell.which('git')) {
      shell.echo('Sorry, this script requires git')
      shell.exit(1)
    }
    shell.config.execPath = shell.which('node').toString()
  }

  absPath (url = []) {
    return this.projectPath && path.join(this.projectPath, ...url)
  }

  gitClone () {
    const tempPath = this.absPath(['temp'])
    fse.emptyDirSync(tempPath)
    // shell.rm('-rf', tempPath)
    // shell.mkdir(tempPath)
    shell.cd(tempPath)
    const res = shell.exec('git clone ' + this.remoteRepositoryPath)
    if (res.code === 0) {
      return Promise.resolve(tempPath)
    } else {
      return Promise.reject(tempPath)
    }
    // this.commitHash = shell.exec('git log -n1 --format=format:"%H"').toString().trim()
  }

  mergeFile (targetPath, sourcePath) {
    // let targetContent = fs.readFileSync(targetPath, 'utf-8').toString()
    // let sourceContent = fs.readFileSync(sourcePath, 'utf-8').toString()
    let targetContent = shell.cat(targetPath)
    let sourceContent = shell.cat(sourcePath)
    if (targetContent) {
      targetContent = JSON.parse(targetContent)
    }
    if (sourceContent) {
      sourceContent = JSON.parse(sourceContent)
    }
    const mergeContent = _.merge({}, sourceContent, targetContent)
    // fs.writeFileSync(targetPath, JSON.stringify(mergeContent, null, '\t'))
    shell.ShellString(JSON.stringify(mergeContent, null, '\t')).to(targetPath)
  }

  download (opt) {
    if (!this.projectPath) {
      return Promise.resolve(false)
    }
    if (opt === 'init') {
      const initPath = this.absPath()
      // shell.rm('-rf', initPath + '/*')
      return this.gitClone().then(tempPath => {
        const sourcePath = path.resolve(tempPath, this.repositoryName, '*')
        const sourcePath2 = path.resolve(tempPath, this.repositoryName, '.*')
        shell.cp('-R', sourcePath, sourcePath2, initPath)
        shell.rm('-rf', tempPath)
        return Promise.resolve(true)
      }).catch((e) => {
        return Promise.resolve(false)
      })
    } else {
      if (!_.isEmpty(this.copyFile)) {
        return this.gitClone().then(tempPath => {
          for (const source in this.copyFile) {
            const sourcePath = path.join(tempPath, this.repositoryName, source)
            const targetPath = this.absPath([this.copyFile[source]])
            console.log('sourcePath', sourcePath)
            console.log('targetPath', targetPath)
            if (source.match(/\.json$/)) {
              if (fse.pathExistsSync(targetPath)) {
                this.mergeFile(targetPath, sourcePath)
              } else {
                // fse.outputJsonSync(targetPath, {})
                fse.copySync(sourcePath, targetPath)
                // this.mergeFile(targetPath, sourcePath)
              }
              // console.log('fse.pathExistsSync(targetPath)', fse.pathExistsSync(targetPath))
              // if (fse.pathExistsSync(targetPath)) {
              //   this.mergeFile(targetPath, sourcePath)
              // } else {
              // }
              // if (targetPath.match(/\.json$/)) {
              //   shell.touch(targetPath)
              //   shell.mv(sourcePath, targetPath)
              // } else {
              //   this.mergeFile(targetPath, sourcePath)
              // }
            } else if (source.match(/\.js$/) || source.match(/\.css$/) || source.match(/\.html$/) || source.match(/\.md$/)) {
              if (!fse.pathExistsSync(targetPath)) {
                fse.outputFileSync(targetPath, '')
              }
              shell.mv(sourcePath, targetPath)
              // fse.moveSync(sourcePath, targetPath)
              // if (targetPath.match(/\.js$/) || targetPath.match(/\.css$/) || targetPath.match(/\.html$/)) {
              //   shell.touch(targetPath)
              // } else {
              //   shell.mkdir('-p', targetPath)
              // }
              // shell.mv(sourcePath, targetPath)
            } else {
              // const copyPath = path.join(tempPath, source)
              // utils.move(copyPath, this.targetFolder)
              // shell.rm('-rf', targetPath + '/*')
              shell.mkdir('-p', targetPath)
              // shell.mv(sourcePath, targetPath)
              // fse.copySync(sourcePath, targetPath)
              shell.cp('-R', sourcePath, targetPath)
              // shell.mv(copyPath, targetFolder)
            }
          }
          shell.rm('-rf', tempPath)
          return Promise.resolve(true)
        })
      } else {
        return Promise.resolve(false)
      }
    }
  }
}

module.exports = GitClone
